# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import additional_application_info
from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.common.types import dates
from google.ads.googleads.v22.enums.types import frequency_cap_time_unit
from google.ads.googleads.v22.enums.types import reach_plan_age_range
from google.ads.googleads.v22.enums.types import (
    reach_plan_conversion_rate_model,
)
from google.ads.googleads.v22.enums.types import reach_plan_network
from google.ads.googleads.v22.enums.types import (
    reach_plan_plannable_user_list_status,
)
from google.ads.googleads.v22.enums.types import reach_plan_surface
from google.ads.googleads.v22.enums.types import target_frequency_time_unit
from google.ads.googleads.v22.enums.types import user_interest_taxonomy_type
from google.ads.googleads.v22.enums.types import (
    user_list_crm_data_source_type as gage_user_list_crm_data_source_type,
)
from google.ads.googleads.v22.enums.types import (
    user_list_type as gage_user_list_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "GenerateConversionRatesRequest",
        "GenerateConversionRatesResponse",
        "ConversionRateSuggestion",
        "ListPlannableLocationsRequest",
        "ListPlannableLocationsResponse",
        "PlannableLocation",
        "ListPlannableProductsRequest",
        "ListPlannableProductsResponse",
        "ProductMetadata",
        "ListPlannableUserListsRequest",
        "ListPlannableUserListsResponse",
        "PlannableUserList",
        "PlannableUserListMetadata",
        "PlannableTargeting",
        "ListPlannableUserInterestsRequest",
        "ListPlannableUserInterestsResponse",
        "PlannableUserInterest",
        "GenerateReachForecastRequest",
        "EffectiveFrequencyLimit",
        "FrequencyCap",
        "Targeting",
        "CampaignDuration",
        "PlannedProduct",
        "GenerateReachForecastResponse",
        "ReachCurve",
        "ReachForecast",
        "Forecast",
        "PlannedProductReachForecast",
        "PlannedProductForecast",
        "OnTargetAudienceMetrics",
        "EffectiveFrequencyBreakdown",
        "ForecastMetricOptions",
        "AudienceTargeting",
        "AdvancedProductTargeting",
        "YouTubeSelectSettings",
        "YouTubeSelectLineUp",
        "SurfaceTargetingCombinations",
        "SurfaceTargeting",
        "TargetFrequencySettings",
    },
)


class GenerateConversionRatesRequest(proto.Message):
    r"""Request message for
    [ReachPlanService.GenerateConversionRates][google.ads.googleads.v22.services.ReachPlanService.GenerateConversionRates].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer. A
            conversion rate based on the historical data of
            this customer may be suggested.
        customer_reach_group (str):
            The name of the customer being planned for.
            This is a user-defined value.

            This field is a member of `oneof`_ ``_customer_reach_group``.
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_reach_group: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class GenerateConversionRatesResponse(proto.Message):
    r"""Response message for
    [ReachPlanService.GenerateConversionRates][google.ads.googleads.v22.services.ReachPlanService.GenerateConversionRates],
    containing conversion rate suggestions for supported plannable
    products.

    Attributes:
        conversion_rate_suggestions (MutableSequence[google.ads.googleads.v22.services.types.ConversionRateSuggestion]):
            A list containing conversion rate
            suggestions. Each repeated element will have an
            associated product code. Multiple suggestions
            may share the same product code.
    """

    conversion_rate_suggestions: MutableSequence["ConversionRateSuggestion"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="ConversionRateSuggestion",
        )
    )


class ConversionRateSuggestion(proto.Message):
    r"""A conversion rate suggestion.

    Attributes:
        conversion_rate_model (google.ads.googleads.v22.enums.types.ReachPlanConversionRateModelEnum.ReachPlanConversionRateModel):
            Model type used to calculate the suggested
            conversion rate.
        plannable_product_code (str):
            The code associated with the plannable product (for example:
            DEMAND_GEN). To list all plannable product codes, use
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].
        conversion_rate (float):
            The suggested conversion rate. The value is
            between 0 and 1 (exclusive).
    """

    conversion_rate_model: (
        reach_plan_conversion_rate_model.ReachPlanConversionRateModelEnum.ReachPlanConversionRateModel
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=reach_plan_conversion_rate_model.ReachPlanConversionRateModelEnum.ReachPlanConversionRateModel,
    )
    plannable_product_code: str = proto.Field(
        proto.STRING,
        number=2,
    )
    conversion_rate: float = proto.Field(
        proto.DOUBLE,
        number=3,
    )


class ListPlannableLocationsRequest(proto.Message):
    r"""Request message for
    [ReachPlanService.ListPlannableLocations][google.ads.googleads.v22.services.ReachPlanService.ListPlannableLocations].

    Attributes:
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class ListPlannableLocationsResponse(proto.Message):
    r"""The list of plannable locations.

    Attributes:
        plannable_locations (MutableSequence[google.ads.googleads.v22.services.types.PlannableLocation]):
            The list of locations available for planning.
            See
            https://developers.google.com/google-ads/api/reference/data/geotargets
            for sample locations.
    """

    plannable_locations: MutableSequence["PlannableLocation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="PlannableLocation",
        )
    )


class PlannableLocation(proto.Message):
    r"""A plannable location: country, metro region, province, etc.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        id (str):
            The location identifier.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The unique location name in English.

            This field is a member of `oneof`_ ``_name``.
        parent_country_id (int):
            The parent country (not present if location is a country).
            If present, will always be a GeoTargetConstant ID.
            Additional information such as country name is provided by
            [ReachPlanService.ListPlannableLocations][google.ads.googleads.v22.services.ReachPlanService.ListPlannableLocations]
            or GoogleAdsService.Search/SearchStream.

            This field is a member of `oneof`_ ``_parent_country_id``.
        country_code (str):
            The ISO-3166-1 alpha-2 country code that is
            associated with the location.

            This field is a member of `oneof`_ ``_country_code``.
        location_type (str):
            The location's type. Location types correspond to
            target_type returned by searching location type in
            GoogleAdsService.Search/SearchStream.

            This field is a member of `oneof`_ ``_location_type``.
    """

    id: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    parent_country_id: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    location_type: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )


class ListPlannableProductsRequest(proto.Message):
    r"""Request to list available products in a given location.

    Attributes:
        plannable_location_id (str):
            Required. The ID of the selected location for planning. To
            list the available plannable location IDs use
            [ReachPlanService.ListPlannableLocations][google.ads.googleads.v22.services.ReachPlanService.ListPlannableLocations].
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    plannable_location_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class ListPlannableProductsResponse(proto.Message):
    r"""A response with all available products.

    Attributes:
        product_metadata (MutableSequence[google.ads.googleads.v22.services.types.ProductMetadata]):
            The list of products available for planning
            and related targeting metadata.
    """

    product_metadata: MutableSequence["ProductMetadata"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ProductMetadata",
    )


class ProductMetadata(proto.Message):
    r"""The metadata associated with an available plannable product.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        plannable_product_code (str):
            The code associated with the ad product (for example:
            BUMPER, TRUEVIEW_IN_STREAM). To list the available plannable
            product codes use
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].

            This field is a member of `oneof`_ ``_plannable_product_code``.
        plannable_product_name (str):
            The name associated with the ad product.
        plannable_targeting (google.ads.googleads.v22.services.types.PlannableTargeting):
            The allowed plannable targeting for this
            product.
    """

    plannable_product_code: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    plannable_product_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    plannable_targeting: "PlannableTargeting" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="PlannableTargeting",
    )


class ListPlannableUserListsRequest(proto.Message):
    r"""Request message for
    [ReachPlanService.ListPlannableUserLists][google.ads.googleads.v22.services.ReachPlanService.ListPlannableUserLists]
    that lists the available user lists for a customer.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_reach_group (str):
            The name of the customer being planned for.
            This is a user-defined value.

            This field is a member of `oneof`_ ``_customer_reach_group``.
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_reach_group: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class ListPlannableUserListsResponse(proto.Message):
    r"""A response with all available user lists with their plannable
    status.

    Attributes:
        plannable_user_lists (MutableSequence[google.ads.googleads.v22.services.types.PlannableUserList]):
            The list of user lists available for planning
            and related targeting metadata.
    """

    plannable_user_lists: MutableSequence["PlannableUserList"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="PlannableUserList",
        )
    )


class PlannableUserList(proto.Message):
    r"""A plannable user list.

    Attributes:
        user_list_info (google.ads.googleads.v22.common.types.UserListInfo):
            The user list ID.
        display_name (str):
            The name of the user list.
        user_list_type (google.ads.googleads.v22.enums.types.UserListTypeEnum.UserListType):
            The user list type.
        plannable_status (google.ads.googleads.v22.enums.types.ReachPlanPlannableUserListStatusEnum.ReachPlanPlannableUserListStatus):
            The plannable status of the user list.
        plannable_user_list_metadata (google.ads.googleads.v22.services.types.PlannableUserListMetadata):
            The relevant metadata for this user list.
    """

    user_list_info: criteria.UserListInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=criteria.UserListInfo,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    user_list_type: gage_user_list_type.UserListTypeEnum.UserListType = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=gage_user_list_type.UserListTypeEnum.UserListType,
        )
    )
    plannable_status: (
        reach_plan_plannable_user_list_status.ReachPlanPlannableUserListStatusEnum.ReachPlanPlannableUserListStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=reach_plan_plannable_user_list_status.ReachPlanPlannableUserListStatusEnum.ReachPlanPlannableUserListStatus,
    )
    plannable_user_list_metadata: "PlannableUserListMetadata" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="PlannableUserListMetadata",
    )


class PlannableUserListMetadata(proto.Message):
    r"""The metadata associated with a plannable user list.

    Attributes:
        user_list_crm_data_source_type (google.ads.googleads.v22.enums.types.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType):
            The data source type of a CRM based user
            list.
    """

    user_list_crm_data_source_type: (
        gage_user_list_crm_data_source_type.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=gage_user_list_crm_data_source_type.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType,
    )


class PlannableTargeting(proto.Message):
    r"""The targeting for which traffic metrics will be reported.

    Attributes:
        age_ranges (MutableSequence[google.ads.googleads.v22.enums.types.ReachPlanAgeRangeEnum.ReachPlanAgeRange]):
            Allowed plannable age ranges for the product
            for which metrics will be reported. Actual
            targeting is computed by mapping this age range
            onto standard Google common.AgeRangeInfo values.
        genders (MutableSequence[google.ads.googleads.v22.common.types.GenderInfo]):
            Targetable genders for the ad product.
        devices (MutableSequence[google.ads.googleads.v22.common.types.DeviceInfo]):
            Targetable devices for the ad product. TABLET device
            targeting is automatically applied to reported metrics when
            MOBILE targeting is selected for CPM_MASTHEAD,
            GOOGLE_PREFERRED_BUMPER, and GOOGLE_PREFERRED_SHORT
            products.
        networks (MutableSequence[google.ads.googleads.v22.enums.types.ReachPlanNetworkEnum.ReachPlanNetwork]):
            Targetable networks for the ad product.
        youtube_select_lineups (MutableSequence[google.ads.googleads.v22.services.types.YouTubeSelectLineUp]):
            Targetable YouTube Select Lineups for the ad
            product.
        surface_targeting (google.ads.googleads.v22.services.types.SurfaceTargetingCombinations):
            Targetable surface combinations for the ad
            product.
    """

    age_ranges: MutableSequence[
        reach_plan_age_range.ReachPlanAgeRangeEnum.ReachPlanAgeRange
    ] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=reach_plan_age_range.ReachPlanAgeRangeEnum.ReachPlanAgeRange,
    )
    genders: MutableSequence[criteria.GenderInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=criteria.GenderInfo,
    )
    devices: MutableSequence[criteria.DeviceInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=criteria.DeviceInfo,
    )
    networks: MutableSequence[
        reach_plan_network.ReachPlanNetworkEnum.ReachPlanNetwork
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=reach_plan_network.ReachPlanNetworkEnum.ReachPlanNetwork,
    )
    youtube_select_lineups: MutableSequence["YouTubeSelectLineUp"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=5,
            message="YouTubeSelectLineUp",
        )
    )
    surface_targeting: "SurfaceTargetingCombinations" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="SurfaceTargetingCombinations",
    )


class ListPlannableUserInterestsRequest(proto.Message):
    r"""Request message for
    [ReachPlanService.ListPlannableUserInterests][google.ads.googleads.v22.services.ReachPlanService.ListPlannableUserInterests].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        user_interest_taxonomy_types (MutableSequence[google.ads.googleads.v22.enums.types.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType]):
            Optional. A filter by user interest type. If set, only user
            interests with a type listed in the filter will be returned.
            If not set, user interests of all supported types will be
            returned. Supported user interest types are AFFINITY and
            IN_MARKET. Each type must be specified at most once.
        name_query (str):
            A filter by user interest name. If set, only
            user interests with a name containing the
            literal string (case insensitive) in the filter
            will be returned. Maximum length is 200
            characters.

            This field is a member of `oneof`_ ``_name_query``.
        path_query (str):
            A filter by user interest path. If set, only
            user interests with a path containing the
            literal string (case insensitive) in the filter
            will be returned. Maximum length is 200
            characters.

            This field is a member of `oneof`_ ``_path_query``.
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    user_interest_taxonomy_types: MutableSequence[
        user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType,
    )
    name_query: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    path_query: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=5,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class ListPlannableUserInterestsResponse(proto.Message):
    r"""Response message for
    [ReachPlanService.ListPlannableUserInterests][google.ads.googleads.v22.services.ReachPlanService.ListPlannableUserInterests].

    Attributes:
        plannable_user_interests (MutableSequence[google.ads.googleads.v22.services.types.PlannableUserInterest]):
            The list of plannable user interests.
    """

    plannable_user_interests: MutableSequence["PlannableUserInterest"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="PlannableUserInterest",
        )
    )


class PlannableUserInterest(proto.Message):
    r"""A plannable user interest that can be targeted in a reach forecast
    using
    [ReachPlanService.GenerateReachForecast][google.ads.googleads.v22.services.ReachPlanService.GenerateReachForecast].

    Attributes:
        user_interest (google.ads.googleads.v22.common.types.UserInterestInfo):
            The user interest id.
        user_interest_type (google.ads.googleads.v22.enums.types.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType):
            The user interest type.
        user_interest_display_name (str):
            The user interest display name.
            For example, "Autos & Vehicles".
        user_interest_path (str):
            The user interest path.
            For example, "/Autos & Vehicles/Motor
            Vehicles/Motor Vehicles by Type/Off-Road
            Vehicles".
    """

    user_interest: criteria.UserInterestInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=criteria.UserInterestInfo,
    )
    user_interest_type: (
        user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType,
    )
    user_interest_display_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    user_interest_path: str = proto.Field(
        proto.STRING,
        number=4,
    )


class GenerateReachForecastRequest(proto.Message):
    r"""Request message for
    [ReachPlanService.GenerateReachForecast][google.ads.googleads.v22.services.ReachPlanService.GenerateReachForecast].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        currency_code (str):
            The currency code.
            Three-character ISO 4217 currency code.

            This field is a member of `oneof`_ ``_currency_code``.
        campaign_duration (google.ads.googleads.v22.services.types.CampaignDuration):
            Required. Campaign duration.
        cookie_frequency_cap (int):
            Chosen cookie frequency cap to be applied to each planned
            product. This is equivalent to the frequency cap exposed in
            Google Ads when creating a campaign, it represents the
            maximum number of times an ad can be shown to the same user.
            If not specified, no cap is applied.

            This field is deprecated in v4 and will eventually be
            removed. Use cookie_frequency_cap_setting instead.

            This field is a member of `oneof`_ ``_cookie_frequency_cap``.
        cookie_frequency_cap_setting (google.ads.googleads.v22.services.types.FrequencyCap):
            Chosen cookie frequency cap to be applied to each planned
            product. This is equivalent to the frequency cap exposed in
            Google Ads when creating a campaign, it represents the
            maximum number of times an ad can be shown to the same user
            during a specified time interval. If not specified, a
            default of 0 (no cap) is applied.

            This field replaces the deprecated cookie_frequency_cap
            field.
        min_effective_frequency (int):
            Chosen minimum effective frequency (the number of times a
            person was exposed to the ad) for the reported reach metrics
            [1-10]. This won't affect the targeting, but just the
            reporting. If not specified, a default of 1 is applied.

            This field cannot be combined with the
            effective_frequency_limit field.

            This field is a member of `oneof`_ ``_min_effective_frequency``.
        effective_frequency_limit (google.ads.googleads.v22.services.types.EffectiveFrequencyLimit):
            The highest minimum effective frequency (the number of times
            a person was exposed to the ad) value [1-10] to include in
            Forecast.effective_frequency_breakdowns. If not specified,
            Forecast.effective_frequency_breakdowns will not be
            provided.

            The effective frequency value provided here will also be
            used as the minimum effective frequency for the reported
            reach metrics.

            This field cannot be combined with the
            min_effective_frequency field.

            This field is a member of `oneof`_ ``_effective_frequency_limit``.
        targeting (google.ads.googleads.v22.services.types.Targeting):
            The targeting to be applied to all products
            selected in the product mix.
            This is planned targeting: execution details
            might vary based on the advertising product,
            consult an implementation specialist.

            See specific metrics for details on how
            targeting affects them.
        planned_products (MutableSequence[google.ads.googleads.v22.services.types.PlannedProduct]):
            Required. The products to be forecast.
            The max number of allowed planned products is
            15.
        forecast_metric_options (google.ads.googleads.v22.services.types.ForecastMetricOptions):
            Controls the forecast metrics returned in the
            response.
        customer_reach_group (str):
            The name of the customer being planned for.
            This is a user-defined value.

            This field is a member of `oneof`_ ``_customer_reach_group``.
        reach_application_info (google.ads.googleads.v22.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    campaign_duration: "CampaignDuration" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="CampaignDuration",
    )
    cookie_frequency_cap: int = proto.Field(
        proto.INT32,
        number=10,
        optional=True,
    )
    cookie_frequency_cap_setting: "FrequencyCap" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="FrequencyCap",
    )
    min_effective_frequency: int = proto.Field(
        proto.INT32,
        number=11,
        optional=True,
    )
    effective_frequency_limit: "EffectiveFrequencyLimit" = proto.Field(
        proto.MESSAGE,
        number=12,
        optional=True,
        message="EffectiveFrequencyLimit",
    )
    targeting: "Targeting" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="Targeting",
    )
    planned_products: MutableSequence["PlannedProduct"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="PlannedProduct",
    )
    forecast_metric_options: "ForecastMetricOptions" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="ForecastMetricOptions",
    )
    customer_reach_group: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    reach_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=15,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class EffectiveFrequencyLimit(proto.Message):
    r"""Effective frequency limit.

    Attributes:
        effective_frequency_breakdown_limit (int):
            The highest effective frequency value to include in
            Forecast.effective_frequency_breakdowns. This field supports
            frequencies 1-10, inclusive.
    """

    effective_frequency_breakdown_limit: int = proto.Field(
        proto.INT32,
        number=1,
    )


class FrequencyCap(proto.Message):
    r"""A rule specifying the maximum number of times an ad can be
    shown to a user over a particular time period.

    Attributes:
        impressions (int):
            Required. The number of impressions,
            inclusive.
        time_unit (google.ads.googleads.v22.enums.types.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit):
            Required. The type of time unit.
    """

    impressions: int = proto.Field(
        proto.INT32,
        number=3,
    )
    time_unit: (
        frequency_cap_time_unit.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=frequency_cap_time_unit.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit,
    )


class Targeting(proto.Message):
    r"""The targeting for which traffic metrics will be reported.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        plannable_location_id (str):
            The ID of the selected location. Plannable location IDs can
            be obtained from
            [ReachPlanService.ListPlannableLocations][google.ads.googleads.v22.services.ReachPlanService.ListPlannableLocations].

            Requests must set either this field or
            ``plannable_location_ids``.

            This field is deprecated as of V12 and will be removed in a
            future release. Use ``plannable_location_ids`` instead.

            This field is a member of `oneof`_ ``_plannable_location_id``.
        plannable_location_ids (MutableSequence[str]):
            The list of plannable location IDs to target with this
            forecast.

            If more than one ID is provided, all IDs must have the same
            ``parent_country_id``. Planning for more than
            ``parent_county`` is not supported. Plannable location IDs
            and their ``parent_country_id`` can be obtained from
            [ReachPlanService.ListPlannableLocations][google.ads.googleads.v22.services.ReachPlanService.ListPlannableLocations].

            Requests must set either this field or
            ``plannable_location_id``.
        age_range (google.ads.googleads.v22.enums.types.ReachPlanAgeRangeEnum.ReachPlanAgeRange):
            Targeted age range.
            An unset value is equivalent to targeting all
            ages.
        genders (MutableSequence[google.ads.googleads.v22.common.types.GenderInfo]):
            Targeted genders.
            An unset value is equivalent to targeting MALE
            and FEMALE.
        devices (MutableSequence[google.ads.googleads.v22.common.types.DeviceInfo]):
            Targeted devices. If not specified, targets all applicable
            devices. Applicable devices vary by product and region and
            can be obtained from
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].
        network (google.ads.googleads.v22.enums.types.ReachPlanNetworkEnum.ReachPlanNetwork):
            Targetable network for the ad product. If not specified,
            targets all applicable networks. Applicable networks vary by
            product and region and can be obtained from
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].
        audience_targeting (google.ads.googleads.v22.services.types.AudienceTargeting):
            Targeted audiences.
            If not specified, does not target any specific
            audience.
    """

    plannable_location_id: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    plannable_location_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=8,
    )
    age_range: reach_plan_age_range.ReachPlanAgeRangeEnum.ReachPlanAgeRange = (
        proto.Field(
            proto.ENUM,
            number=2,
            enum=reach_plan_age_range.ReachPlanAgeRangeEnum.ReachPlanAgeRange,
        )
    )
    genders: MutableSequence[criteria.GenderInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=criteria.GenderInfo,
    )
    devices: MutableSequence[criteria.DeviceInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=criteria.DeviceInfo,
    )
    network: reach_plan_network.ReachPlanNetworkEnum.ReachPlanNetwork = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=reach_plan_network.ReachPlanNetworkEnum.ReachPlanNetwork,
        )
    )
    audience_targeting: "AudienceTargeting" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="AudienceTargeting",
    )


class CampaignDuration(proto.Message):
    r"""The duration of a planned campaign.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        duration_in_days (int):
            The duration value in days.

            This field cannot be combined with the date_range field.

            This field is a member of `oneof`_ ``_duration_in_days``.
        date_range (google.ads.googleads.v22.common.types.DateRange):
            Date range of the campaign. Dates are in the yyyy-mm-dd
            format and inclusive. The end date must be < 1 year in the
            future and the date range must be <= 92 days long.

            This field cannot be combined with the duration_in_days
            field.
    """

    duration_in_days: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )
    date_range: dates.DateRange = proto.Field(
        proto.MESSAGE,
        number=3,
        message=dates.DateRange,
    )


class PlannedProduct(proto.Message):
    r"""A product being planned for reach.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        plannable_product_code (str):
            Required. Selected product for planning. The code associated
            with the ad product (for example: Trueview, Bumper). To list
            the available plannable product codes use
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].

            This field is a member of `oneof`_ ``_plannable_product_code``.
        budget_micros (int):
            Required. Maximum budget allocation in micros for the
            selected product. The value is specified in the selected
            planning currency_code. For example: 1 000 000$ = 1 000 000
            000 000 micros.

            This field is a member of `oneof`_ ``_budget_micros``.
        conversion_rate (float):
            Conversion rate as a decimal between 0 and 1, exclusive. For
            example: if 2% of ad interactions are expected to lead to
            conversions, conversion_rate should be 0.02. This field is
            required for DEMAND_GEN plannable products. It is not
            supported for other plannable products.

            This field is a member of `oneof`_ ``_conversion_rate``.
        advanced_product_targeting (google.ads.googleads.v22.services.types.AdvancedProductTargeting):
            Targeting settings for the selected product. To list the
            available targeting for each product use
            [ReachPlanService.ListPlannableProducts][google.ads.googleads.v22.services.ReachPlanService.ListPlannableProducts].
    """

    plannable_product_code: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    budget_micros: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    conversion_rate: float = proto.Field(
        proto.DOUBLE,
        number=6,
        optional=True,
    )
    advanced_product_targeting: "AdvancedProductTargeting" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="AdvancedProductTargeting",
    )


class GenerateReachForecastResponse(proto.Message):
    r"""Response message containing the generated reach curve.

    Attributes:
        on_target_audience_metrics (google.ads.googleads.v22.services.types.OnTargetAudienceMetrics):
            Reference on target audiences for this curve.
        reach_curve (google.ads.googleads.v22.services.types.ReachCurve):
            The generated reach curve for the planned
            product mix.
    """

    on_target_audience_metrics: "OnTargetAudienceMetrics" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="OnTargetAudienceMetrics",
    )
    reach_curve: "ReachCurve" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ReachCurve",
    )


class ReachCurve(proto.Message):
    r"""The reach curve for the planned products.

    Attributes:
        reach_forecasts (MutableSequence[google.ads.googleads.v22.services.types.ReachForecast]):
            All points on the reach curve.
    """

    reach_forecasts: MutableSequence["ReachForecast"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ReachForecast",
    )


class ReachForecast(proto.Message):
    r"""A point on reach curve.

    Attributes:
        cost_micros (int):
            The cost in micros.
        forecast (google.ads.googleads.v22.services.types.Forecast):
            Forecasted traffic metrics for this point.
        planned_product_reach_forecasts (MutableSequence[google.ads.googleads.v22.services.types.PlannedProductReachForecast]):
            The forecasted allocation and traffic metrics
            for each planned product at this point on the
            reach curve.
    """

    cost_micros: int = proto.Field(
        proto.INT64,
        number=5,
    )
    forecast: "Forecast" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="Forecast",
    )
    planned_product_reach_forecasts: MutableSequence[
        "PlannedProductReachForecast"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="PlannedProductReachForecast",
    )


class Forecast(proto.Message):
    r"""Forecasted traffic metrics for the planned products and
    targeting.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        on_target_reach (int):
            Number of unique people reached at least
            GenerateReachForecastRequest.min_effective_frequency or
            GenerateReachForecastRequest.effective_frequency_limit times
            that exactly matches the Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the on_target_reach value will be rounded to 0.

            This field is a member of `oneof`_ ``_on_target_reach``.
        total_reach (int):
            Total number of unique people reached at least
            GenerateReachForecastRequest.min_effective_frequency or
            GenerateReachForecastRequest.effective_frequency_limit
            times. This includes people that may fall outside the
            specified Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the total_reach value will be rounded to 0.

            This field is a member of `oneof`_ ``_total_reach``.
        on_target_impressions (int):
            Number of ad impressions that exactly matches
            the Targeting.

            This field is a member of `oneof`_ ``_on_target_impressions``.
        total_impressions (int):
            Total number of ad impressions. This includes
            impressions that may fall outside the specified
            Targeting, due to insufficient information on
            signed-in users.

            This field is a member of `oneof`_ ``_total_impressions``.
        viewable_impressions (int):
            Number of times the ad's impressions were
            considered viewable. See
            https://support.google.com/google-ads/answer/7029393
            for more information about what makes an ad
            viewable and how viewability is measured.

            This field is a member of `oneof`_ ``_viewable_impressions``.
        effective_frequency_breakdowns (MutableSequence[google.ads.googleads.v22.services.types.EffectiveFrequencyBreakdown]):
            A list of effective frequency forecasts. The list is ordered
            starting with 1+ and ending with the value set in
            GenerateReachForecastRequest.effective_frequency_limit. If
            no effective_frequency_limit was set, this list will be
            empty.
        on_target_coview_reach (int):
            Number of unique people reached that exactly
            matches the Targeting including co-viewers.

            This field is a member of `oneof`_ ``_on_target_coview_reach``.
        total_coview_reach (int):
            Number of unique people reached including
            co-viewers. This includes people that may fall
            outside the specified Targeting.

            This field is a member of `oneof`_ ``_total_coview_reach``.
        on_target_coview_impressions (int):
            Number of ad impressions that exactly matches
            the Targeting including co-viewers.

            This field is a member of `oneof`_ ``_on_target_coview_impressions``.
        total_coview_impressions (int):
            Total number of ad impressions including
            co-viewers. This includes impressions that may
            fall outside the specified Targeting, due to
            insufficient information on signed-in users.

            This field is a member of `oneof`_ ``_total_coview_impressions``.
        conversions (float):
            The number of conversions. This metric is only available for
            DEMAND_GEN plannable products.

            See https://support.google.com/google-ads/answer/2375431 for
            more information on conversions.

            This field is a member of `oneof`_ ``_conversions``.
        trueview_views (int):
            Number of ad views forecasted for the
            specified product and targeting. A TrueView View
            is counted when a viewer views a larger portion
            or the entirety of an ad beyond an impression.

            See
            https://support.google.com/google-ads/answer/2375431
            for more information on TrueView Views.

            This field is a member of `oneof`_ ``_trueview_views``.
    """

    on_target_reach: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    total_reach: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    on_target_impressions: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    total_impressions: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    viewable_impressions: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    effective_frequency_breakdowns: MutableSequence[
        "EffectiveFrequencyBreakdown"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message="EffectiveFrequencyBreakdown",
    )
    on_target_coview_reach: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    total_coview_reach: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )
    on_target_coview_impressions: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )
    total_coview_impressions: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )
    conversions: float = proto.Field(
        proto.DOUBLE,
        number=16,
        optional=True,
    )
    trueview_views: int = proto.Field(
        proto.INT64,
        number=17,
        optional=True,
    )


class PlannedProductReachForecast(proto.Message):
    r"""The forecasted allocation and traffic metrics for a specific
    product at a point on the reach curve.

    Attributes:
        plannable_product_code (str):
            Selected product for planning. The product
            codes returned are within the set of the ones
            returned by ListPlannableProducts when using the
            same location ID.
        cost_micros (int):
            The cost in micros. This may differ from the
            product's input allocation if one or more
            planned products cannot fulfill the budget
            because of limited inventory.
        planned_product_forecast (google.ads.googleads.v22.services.types.PlannedProductForecast):
            Forecasted traffic metrics for this product.
    """

    plannable_product_code: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=2,
    )
    planned_product_forecast: "PlannedProductForecast" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="PlannedProductForecast",
    )


class PlannedProductForecast(proto.Message):
    r"""Forecasted traffic metrics for a planned product.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        on_target_reach (int):
            Number of unique people reached that exactly matches the
            Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the on_target_reach value will be rounded to 0.
        total_reach (int):
            Number of unique people reached. This includes people that
            may fall outside the specified Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the total_reach value will be rounded to 0.
        on_target_impressions (int):
            Number of ad impressions that exactly matches
            the Targeting.
        total_impressions (int):
            Total number of ad impressions. This includes
            impressions that may fall outside the specified
            Targeting, due to insufficient information on
            signed-in users.
        viewable_impressions (int):
            Number of times the ad's impressions were
            considered viewable. See
            https://support.google.com/google-ads/answer/7029393
            for more information about what makes an ad
            viewable and how viewability is measured.

            This field is a member of `oneof`_ ``_viewable_impressions``.
        on_target_coview_reach (int):
            Number of unique people reached that exactly
            matches the Targeting including co-viewers.

            This field is a member of `oneof`_ ``_on_target_coview_reach``.
        total_coview_reach (int):
            Number of unique people reached including
            co-viewers. This includes people that may fall
            outside the specified Targeting.

            This field is a member of `oneof`_ ``_total_coview_reach``.
        on_target_coview_impressions (int):
            Number of ad impressions that exactly matches
            the Targeting including co-viewers.

            This field is a member of `oneof`_ ``_on_target_coview_impressions``.
        total_coview_impressions (int):
            Total number of ad impressions including
            co-viewers. This includes impressions that may
            fall outside the specified Targeting, due to
            insufficient information on signed-in users.

            This field is a member of `oneof`_ ``_total_coview_impressions``.
        average_frequency (float):
            The number of times per selected time unit a
            user will see an ad, averaged over the number of
            time units in the forecast length. This field
            will only be populated for a Target Frequency
            campaign.

            See
            https://support.google.com/google-ads/answer/12400225
            for more information about Target Frequency
            campaigns.

            This field is a member of `oneof`_ ``_average_frequency``.
        conversions (float):
            The number of conversions. This metric is only available for
            DEMAND_GEN plannable products.

            See https://support.google.com/google-ads/answer/2375431 for
            more information on conversions.

            This field is a member of `oneof`_ ``_conversions``.
        trueview_views (int):
            Number of ad views forecasted for the
            specified product and targeting. A TrueView View
            is counted when a viewer views a larger portion
            or the entirety of an ad beyond an impression.

            See
            https://support.google.com/google-ads/answer/2375431
            for more information on TrueView Views.

            This field is a member of `oneof`_ ``_trueview_views``.
    """

    on_target_reach: int = proto.Field(
        proto.INT64,
        number=1,
    )
    total_reach: int = proto.Field(
        proto.INT64,
        number=2,
    )
    on_target_impressions: int = proto.Field(
        proto.INT64,
        number=3,
    )
    total_impressions: int = proto.Field(
        proto.INT64,
        number=4,
    )
    viewable_impressions: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    on_target_coview_reach: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    total_coview_reach: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    on_target_coview_impressions: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    total_coview_impressions: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    average_frequency: float = proto.Field(
        proto.DOUBLE,
        number=10,
        optional=True,
    )
    conversions: float = proto.Field(
        proto.DOUBLE,
        number=12,
        optional=True,
    )
    trueview_views: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )


class OnTargetAudienceMetrics(proto.Message):
    r"""Audience metrics for the planned products. These metrics consider
    the following targeting dimensions:

    - Location
    - PlannableAgeRange
    - Gender
    - AudienceTargeting (only for youtube_audience_size)


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        youtube_audience_size (int):
            Reference audience size matching the
            considered targeting for YouTube.

            This field is a member of `oneof`_ ``_youtube_audience_size``.
        census_audience_size (int):
            Reference audience size matching the
            considered targeting for Census.

            This field is a member of `oneof`_ ``_census_audience_size``.
    """

    youtube_audience_size: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    census_audience_size: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )


class EffectiveFrequencyBreakdown(proto.Message):
    r"""A breakdown of the number of unique people reached at a given
    effective frequency.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        effective_frequency (int):
            The effective frequency [1-10].
        on_target_reach (int):
            The number of unique people reached at least
            effective_frequency times that exactly matches the
            Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the on_target_reach value will be rounded to 0.
        total_reach (int):
            Total number of unique people reached at least
            effective_frequency times. This includes people that may
            fall outside the specified Targeting.

            Note that a minimum number of unique people must be reached
            in order for data to be reported. If the minimum number is
            not met, the total_reach value will be rounded to 0.
        effective_coview_reach (int):
            The number of users (including co-viewing users) reached for
            the associated effective_frequency value.

            This field is a member of `oneof`_ ``_effective_coview_reach``.
        on_target_effective_coview_reach (int):
            The number of users (including co-viewing users) reached for
            the associated effective_frequency value within the
            specified plan demographic.

            This field is a member of `oneof`_ ``_on_target_effective_coview_reach``.
    """

    effective_frequency: int = proto.Field(
        proto.INT32,
        number=1,
    )
    on_target_reach: int = proto.Field(
        proto.INT64,
        number=2,
    )
    total_reach: int = proto.Field(
        proto.INT64,
        number=3,
    )
    effective_coview_reach: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    on_target_effective_coview_reach: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )


class ForecastMetricOptions(proto.Message):
    r"""Controls forecast metrics to return.

    Attributes:
        include_coview (bool):
            Indicates whether to include co-view metrics
            in the response forecast.
    """

    include_coview: bool = proto.Field(
        proto.BOOL,
        number=1,
    )


class AudienceTargeting(proto.Message):
    r"""Audience targeting for reach forecast.

    Attributes:
        user_interest (MutableSequence[google.ads.googleads.v22.common.types.UserInterestInfo]):
            List of audiences based on user interests to
            be targeted.
        user_lists (MutableSequence[google.ads.googleads.v22.common.types.UserListInfo]):
            List of audiences based on user lists to be
            targeted.
    """

    user_interest: MutableSequence[criteria.UserInterestInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.UserInterestInfo,
        )
    )
    user_lists: MutableSequence[criteria.UserListInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=criteria.UserListInfo,
    )


class AdvancedProductTargeting(proto.Message):
    r"""Advanced targeting settings for products.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        surface_targeting_settings (google.ads.googleads.v22.services.types.SurfaceTargeting):
            Surface targeting settings for this product.
        target_frequency_settings (google.ads.googleads.v22.services.types.TargetFrequencySettings):
            Settings for a Target frequency campaign. Must be set when
            selecting the TARGET_FREQUENCY product.

            See https://support.google.com/google-ads/answer/12400225
            for more information about Target Frequency campaigns.
        youtube_select_settings (google.ads.googleads.v22.services.types.YouTubeSelectSettings):
            Settings for YouTube Select targeting.

            This field is a member of `oneof`_ ``advanced_targeting``.
    """

    surface_targeting_settings: "SurfaceTargeting" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="SurfaceTargeting",
    )
    target_frequency_settings: "TargetFrequencySettings" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="TargetFrequencySettings",
    )
    youtube_select_settings: "YouTubeSelectSettings" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="advanced_targeting",
        message="YouTubeSelectSettings",
    )


class YouTubeSelectSettings(proto.Message):
    r"""Request settings for YouTube Select Lineups

    Attributes:
        lineup_id (int):
            Lineup for YouTube Select Targeting.
    """

    lineup_id: int = proto.Field(
        proto.INT64,
        number=1,
    )


class YouTubeSelectLineUp(proto.Message):
    r"""A Plannable YouTube Select Lineup for product targeting.

    Attributes:
        lineup_id (int):
            The ID of the YouTube Select Lineup.
        lineup_name (str):
            The unique name of the YouTube Select Lineup.
    """

    lineup_id: int = proto.Field(
        proto.INT64,
        number=1,
    )
    lineup_name: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SurfaceTargetingCombinations(proto.Message):
    r"""The surface targeting combinations available for an ad
    product.

    Attributes:
        default_targeting (google.ads.googleads.v22.services.types.SurfaceTargeting):
            Default surface targeting applied to the ad
            product.
        available_targeting_combinations (MutableSequence[google.ads.googleads.v22.services.types.SurfaceTargeting]):
            Available surface target combinations for the
            ad product.
    """

    default_targeting: "SurfaceTargeting" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="SurfaceTargeting",
    )
    available_targeting_combinations: MutableSequence["SurfaceTargeting"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="SurfaceTargeting",
        )
    )


class SurfaceTargeting(proto.Message):
    r"""Container for surfaces for a product. Surfaces refer to the
    available types of ad inventories such as In-Feed, In-Stream,
    and Shorts.

    Attributes:
        surfaces (MutableSequence[google.ads.googleads.v22.enums.types.ReachPlanSurfaceEnum.ReachPlanSurface]):
            List of surfaces available to target.
    """

    surfaces: MutableSequence[
        reach_plan_surface.ReachPlanSurfaceEnum.ReachPlanSurface
    ] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=reach_plan_surface.ReachPlanSurfaceEnum.ReachPlanSurface,
    )


class TargetFrequencySettings(proto.Message):
    r"""Target Frequency settings for a supported product.

    Attributes:
        time_unit (google.ads.googleads.v22.enums.types.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit):
            Required. The time unit used to describe the time frame for
            target_frequency.
        target_frequency (int):
            Required. The target frequency goal per
            selected time unit.
    """

    time_unit: (
        target_frequency_time_unit.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=target_frequency_time_unit.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit,
    )
    target_frequency: int = proto.Field(
        proto.INT32,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    application_instance as gage_application_instance,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdditionalApplicationInfo",
    },
)


class AdditionalApplicationInfo(proto.Message):
    r"""Additional information about the application/tool issuing the
    request. This field is only used by [ContentCreatorInsightsService],
    [AudienceInsightsService], and [ReachPlanService] APIs.

    Attributes:
        application_id (str):
            The unique identifier of the agency proprietary application.
            This identifier is generated by Google. Reach out to your
            Google representative to request an application_id for each
            new application being integrated.
        application_instance (google.ads.googleads.v21.enums.types.ApplicationInstanceEnum.ApplicationInstance):
            The instance type of the application sending
            the request.
    """

    application_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    application_instance: (
        gage_application_instance.ApplicationInstanceEnum.ApplicationInstance
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_application_instance.ApplicationInstanceEnum.ApplicationInstance,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "OfflineUserDataJobFailureReasonEnum",
    },
)


class OfflineUserDataJobFailureReasonEnum(proto.Message):
    r"""Container for enum describing reasons why an offline user
    data job failed to be processed.

    """

    class OfflineUserDataJobFailureReason(proto.Enum):
        r"""The failure reason of an offline user data job.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            INSUFFICIENT_MATCHED_TRANSACTIONS (2):
                The matched transactions are insufficient.
            INSUFFICIENT_TRANSACTIONS (3):
                The uploaded transactions are insufficient.
            HIGH_AVERAGE_TRANSACTION_VALUE (4):
                The average transaction value is unusually high for your
                account. If this is intended, contact support to request an
                exception. Learn more at
                https://support.google.com/google-ads/answer/10018944#transaction_value
            LOW_AVERAGE_TRANSACTION_VALUE (5):
                The average transaction value is unusually low for your
                account. If this is intended, contact support to request an
                exception. Learn more at
                https://support.google.com/google-ads/answer/10018944#transaction_value
            NEWLY_OBSERVED_CURRENCY_CODE (6):
                There's a currency code that you haven't used before in your
                uploads. If this is intended, contact support to request an
                exception. Learn more at
                https://support.google.com/google-ads/answer/10018944#Unrecognized_currency
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INSUFFICIENT_MATCHED_TRANSACTIONS = 2
        INSUFFICIENT_TRANSACTIONS = 3
        HIGH_AVERAGE_TRANSACTION_VALUE = 4
        LOW_AVERAGE_TRANSACTION_VALUE = 5
        NEWLY_OBSERVED_CURRENCY_CODE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.resources.types import payments_account


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "ListPaymentsAccountsRequest",
        "ListPaymentsAccountsResponse",
    },
)


class ListPaymentsAccountsRequest(proto.Message):
    r"""Request message for fetching all accessible payments
    accounts.

    Attributes:
        customer_id (str):
            Required. The ID of the customer to apply the
            PaymentsAccount list operation to.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListPaymentsAccountsResponse(proto.Message):
    r"""Response message for
    [PaymentsAccountService.ListPaymentsAccounts][google.ads.googleads.v21.services.PaymentsAccountService.ListPaymentsAccounts].

    Attributes:
        payments_accounts (MutableSequence[google.ads.googleads.v21.resources.types.PaymentsAccount]):
            The list of accessible payments accounts.
    """

    payments_accounts: MutableSequence[payments_account.PaymentsAccount] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=payments_account.PaymentsAccount,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetGroupListingGroupFilterErrorEnum",
    },
)


class AssetGroupListingGroupFilterErrorEnum(proto.Message):
    r"""Container for enum describing possible asset group listing
    group filter errors.

    """

    class AssetGroupListingGroupFilterError(proto.Enum):
        r"""Enum describing possible asset group listing group filter
        errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            TREE_TOO_DEEP (2):
                Listing group tree is too deep.
            UNIT_CANNOT_HAVE_CHILDREN (3):
                Listing Group UNIT node cannot have children.
            SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD (4):
                Listing Group SUBDIVISION node must have
                everything else child.
            DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS (5):
                Dimension type of Listing Group must be the
                same as that of its siblings.
            SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS (6):
                The sibling Listing Groups target exactly the
                same dimension value.
            SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS (7):
                The dimension type is the same as one of the
                ancestor Listing Groups.
            MULTIPLE_ROOTS (8):
                Each Listing Group tree must have a single
                root.
            INVALID_DIMENSION_VALUE (9):
                Invalid Listing Group dimension value.
            MUST_REFINE_HIERARCHICAL_PARENT_TYPE (10):
                Hierarchical dimension must refine a
                dimension of the same type.
            INVALID_PRODUCT_BIDDING_CATEGORY (11):
                Invalid Product Bidding Category.
            CHANGING_CASE_VALUE_WITH_CHILDREN (12):
                Modifying case value is allowed only while
                updating the entire subtree at the same time.
            SUBDIVISION_HAS_CHILDREN (13):
                Subdivision node has children which must be
                removed first.
            CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE (14):
                Dimension can't subdivide everything-else
                node in its own hierarchy.
            DIMENSION_TYPE_NOT_ALLOWED (15):
                This dimension type is not allowed in this
                context.
            DUPLICATE_WEBPAGE_FILTER_UNDER_ASSET_GROUP (16):
                All the webpage filters under an AssetGroup
                should be distinct.
            LISTING_SOURCE_NOT_ALLOWED (17):
                Filter of the listing source type is not
                allowed in the context.
            FILTER_EXCLUSION_NOT_ALLOWED (18):
                Exclusion filters are not allowed in the
                context.
            MULTIPLE_LISTING_SOURCES (19):
                All the filters under an AssetGroup should
                have the same listing source.
            MULTIPLE_WEBPAGE_CONDITION_TYPES_NOT_ALLOWED (20):
                All the conditions in a webpage needs to be
                of same type.
            MULTIPLE_WEBPAGE_TYPES_PER_ASSET_GROUP (21):
                All the webpage types of the filters under an AssetGroup
                should be of same type. Example: All the webpage types can
                be of type custom_label or url_contains but not both.
            PAGE_FEED_FILTER_HAS_PARENT (22):
                All page feed filter nodes are root nodes and
                they can't have a parent.
            MULTIPLE_OPERATIONS_ON_ONE_NODE (23):
                There cannot be more than one mutate
                operation per request that targets a single
                asset group listing group filter.
            TREE_WAS_INVALID_BEFORE_MUTATION (24):
                The tree is in an invalid state in the
                database. Any changes that don't fix its issues
                will fail validation.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        TREE_TOO_DEEP = 2
        UNIT_CANNOT_HAVE_CHILDREN = 3
        SUBDIVISION_MUST_HAVE_EVERYTHING_ELSE_CHILD = 4
        DIFFERENT_DIMENSION_TYPE_BETWEEN_SIBLINGS = 5
        SAME_DIMENSION_VALUE_BETWEEN_SIBLINGS = 6
        SAME_DIMENSION_TYPE_BETWEEN_ANCESTORS = 7
        MULTIPLE_ROOTS = 8
        INVALID_DIMENSION_VALUE = 9
        MUST_REFINE_HIERARCHICAL_PARENT_TYPE = 10
        INVALID_PRODUCT_BIDDING_CATEGORY = 11
        CHANGING_CASE_VALUE_WITH_CHILDREN = 12
        SUBDIVISION_HAS_CHILDREN = 13
        CANNOT_REFINE_HIERARCHICAL_EVERYTHING_ELSE = 14
        DIMENSION_TYPE_NOT_ALLOWED = 15
        DUPLICATE_WEBPAGE_FILTER_UNDER_ASSET_GROUP = 16
        LISTING_SOURCE_NOT_ALLOWED = 17
        FILTER_EXCLUSION_NOT_ALLOWED = 18
        MULTIPLE_LISTING_SOURCES = 19
        MULTIPLE_WEBPAGE_CONDITION_TYPES_NOT_ALLOWED = 20
        MULTIPLE_WEBPAGE_TYPES_PER_ASSET_GROUP = 21
        PAGE_FEED_FILTER_HAS_PARENT = 22
        MULTIPLE_OPERATIONS_ON_ONE_NODE = 23
        TREE_WAS_INVALID_BEFORE_MUTATION = 24


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "HotelPriceBucketEnum",
    },
)


class HotelPriceBucketEnum(proto.Message):
    r"""Container for enum describing hotel price bucket for a hotel
    itinerary.

    """

    class HotelPriceBucket(proto.Enum):
        r"""Enum describing possible hotel price buckets.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            LOWEST_UNIQUE (2):
                Uniquely lowest price. Partner has the lowest
                price, and no other partners are within a small
                variance of that price.
            LOWEST_TIED (3):
                Tied for lowest price. Partner is within a
                small variance of the lowest price.
            NOT_LOWEST (4):
                Not lowest price. Partner is not within a
                small variance of the lowest price.
            ONLY_PARTNER_SHOWN (5):
                Partner was the only one shown.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LOWEST_UNIQUE = 2
        LOWEST_TIED = 3
        NOT_LOWEST = 4
        ONLY_PARTNER_SHOWN = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.resources.types import user_list_customer_type
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "MutateUserListCustomerTypesRequest",
        "UserListCustomerTypeOperation",
        "MutateUserListCustomerTypesResponse",
        "MutateUserListCustomerTypeResult",
    },
)


class MutateUserListCustomerTypesRequest(proto.Message):
    r"""Request message for
    [UserListCustomerTypeService.MutateUserListCustomerTypes][google.ads.googleads.v20.services.UserListCustomerTypeService.MutateUserListCustomerTypes].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose user
            list customer types are being modified.
        operations (MutableSequence[google.ads.googleads.v20.services.types.UserListCustomerTypeOperation]):
            Required. The list of operations to perform
            on the user list customer types.
        partial_failure (bool):
            Optional. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid. Default is false.
        validate_only (bool):
            Optional. If true, the request is validated
            but not executed. Only errors are returned, not
            results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["UserListCustomerTypeOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="UserListCustomerTypeOperation",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class UserListCustomerTypeOperation(proto.Message):
    r"""A single mutate operation on the user list customer type.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v20.resources.types.UserListCustomerType):
            Attach a user list customer type to a user
            list. No resource name is expected for the new
            user list customer type.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove an existing user list customer type. A resource name
            for the removed user list customer type is expected, in this
            format:

            ``customers/{customer_id}/userListCustomerTypes/{user_list_id}~{customer_type_category}``

            This field is a member of `oneof`_ ``operation``.
    """

    create: user_list_customer_type.UserListCustomerType = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=user_list_customer_type.UserListCustomerType,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="operation",
    )


class MutateUserListCustomerTypesResponse(proto.Message):
    r"""Response message for a user list customer type mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v20.services.types.MutateUserListCustomerTypeResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateUserListCustomerTypeResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="MutateUserListCustomerTypeResult",
        )
    )


class MutateUserListCustomerTypeResult(proto.Message):
    r"""The result for the user list customer type mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "FeedItemSetErrorEnum",
    },
)


class FeedItemSetErrorEnum(proto.Message):
    r"""Container for enum describing possible feed item set errors."""

    class FeedItemSetError(proto.Enum):
        r"""Enum describing possible feed item set errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FEED_ITEM_SET_REMOVED (2):
                The given ID refers to a removed FeedItemSet.
            CANNOT_CLEAR_DYNAMIC_FILTER (3):
                The dynamic filter of a feed item set cannot
                be cleared on UPDATE if it exists. A set is
                either static or dynamic once added, and that
                cannot change.
            CANNOT_CREATE_DYNAMIC_FILTER (4):
                The dynamic filter of a feed item set cannot
                be created on UPDATE if it does not exist. A set
                is either static or dynamic once added, and that
                cannot change.
            INVALID_FEED_TYPE (5):
                FeedItemSets can only be made for location or
                affiliate location feeds.
            DUPLICATE_NAME (6):
                FeedItemSets duplicate name. Name should be
                unique within an account.
            WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE (7):
                The feed type of the parent Feed is not compatible with the
                type of dynamic filter being set. For example, you can only
                set dynamic_location_set_filter for LOCATION feed item sets.
            DYNAMIC_FILTER_INVALID_CHAIN_IDS (8):
                Chain ID specified for
                AffiliateLocationFeedData is invalid.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FEED_ITEM_SET_REMOVED = 2
        CANNOT_CLEAR_DYNAMIC_FILTER = 3
        CANNOT_CREATE_DYNAMIC_FILTER = 4
        INVALID_FEED_TYPE = 5
        DUPLICATE_NAME = 6
        WRONG_DYNAMIC_FILTER_FOR_FEED_TYPE = 7
        DYNAMIC_FILTER_INVALID_CHAIN_IDS = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import customer_match_upload_key_type
from google.ads.googleads.v20.enums.types import lookalike_expansion_level
from google.ads.googleads.v20.enums.types import user_list_crm_data_source_type
from google.ads.googleads.v20.enums.types import (
    user_list_date_rule_item_operator,
)
from google.ads.googleads.v20.enums.types import (
    user_list_flexible_rule_operator,
)
from google.ads.googleads.v20.enums.types import user_list_logical_rule_operator
from google.ads.googleads.v20.enums.types import (
    user_list_number_rule_item_operator,
)
from google.ads.googleads.v20.enums.types import user_list_prepopulation_status
from google.ads.googleads.v20.enums.types import user_list_rule_type
from google.ads.googleads.v20.enums.types import (
    user_list_string_rule_item_operator,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "LookalikeUserListInfo",
        "SimilarUserListInfo",
        "CrmBasedUserListInfo",
        "UserListRuleInfo",
        "UserListRuleItemGroupInfo",
        "UserListRuleItemInfo",
        "UserListDateRuleItemInfo",
        "UserListNumberRuleItemInfo",
        "UserListStringRuleItemInfo",
        "FlexibleRuleOperandInfo",
        "FlexibleRuleUserListInfo",
        "RuleBasedUserListInfo",
        "LogicalUserListInfo",
        "UserListLogicalRuleInfo",
        "LogicalUserListOperandInfo",
        "BasicUserListInfo",
        "UserListActionInfo",
    },
)


class LookalikeUserListInfo(proto.Message):
    r"""LookalikeUserlist, composed of users similar to those
    of a configurable seed (set of UserLists)

    Attributes:
        seed_user_list_ids (MutableSequence[int]):
            Seed UserList ID from which this list is
            derived, provided by user.
        expansion_level (google.ads.googleads.v20.enums.types.LookalikeExpansionLevelEnum.LookalikeExpansionLevel):
            Expansion level, reflecting the size of the
            lookalike audience
        country_codes (MutableSequence[str]):
            Countries targeted by the Lookalike.
            Two-letter country code as defined by ISO-3166
    """

    seed_user_list_ids: MutableSequence[int] = proto.RepeatedField(
        proto.INT64,
        number=1,
    )
    expansion_level: (
        lookalike_expansion_level.LookalikeExpansionLevelEnum.LookalikeExpansionLevel
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=lookalike_expansion_level.LookalikeExpansionLevelEnum.LookalikeExpansionLevel,
    )
    country_codes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class SimilarUserListInfo(proto.Message):
    r"""SimilarUserList is a list of users which are similar to users
    from another UserList. These lists are read-only and
    automatically created by Google.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        seed_user_list (str):
            Seed UserList from which this list is
            derived.

            This field is a member of `oneof`_ ``_seed_user_list``.
    """

    seed_user_list: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class CrmBasedUserListInfo(proto.Message):
    r"""UserList of CRM users provided by the advertiser.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        app_id (str):
            A string that uniquely identifies a mobile
            application from which the data was collected.
            For iOS, the ID string is the 9 digit string
            that appears at the end of an App Store URL (for
            example, "476943146" for "Flood-It! 2" whose App
            Store link is
            http://itunes.apple.com/us/app/flood-it!-2/id476943146).
            For Android, the ID string is the application's
            package name (for example,
            "com.labpixies.colordrips" for "Color Drips"
            given Google Play link
            https://play.google.com/store/apps/details?id=com.labpixies.colordrips).
            Required when creating CrmBasedUserList for
            uploading mobile advertising IDs.

            This field is a member of `oneof`_ ``_app_id``.
        upload_key_type (google.ads.googleads.v20.enums.types.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType):
            Matching key type of the list.
            Mixed data types are not allowed on the same
            list. This field is required for an ADD
            operation.
        data_source_type (google.ads.googleads.v20.enums.types.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType):
            Data source of the list. Default value is FIRST_PARTY. Only
            customers on the allow-list can create third-party sourced
            CRM lists.
    """

    app_id: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    upload_key_type: (
        customer_match_upload_key_type.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=customer_match_upload_key_type.CustomerMatchUploadKeyTypeEnum.CustomerMatchUploadKeyType,
    )
    data_source_type: (
        user_list_crm_data_source_type.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=user_list_crm_data_source_type.UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType,
    )


class UserListRuleInfo(proto.Message):
    r"""A client defined rule based on custom parameters sent by web
    sites or uploaded by the advertiser.

    Attributes:
        rule_type (google.ads.googleads.v20.enums.types.UserListRuleTypeEnum.UserListRuleType):
            Rule type is used to determine how to group
            rule items.
            The default is OR of ANDs (disjunctive normal
            form). That is, rule items will be ANDed
            together within rule item groups and the groups
            themselves will be ORed together.

            OR of ANDs is the only supported type for
            FlexibleRuleUserList.
        rule_item_groups (MutableSequence[google.ads.googleads.v20.common.types.UserListRuleItemGroupInfo]):
            List of rule item groups that defines this rule. Rule item
            groups are grouped together based on rule_type.
    """

    rule_type: user_list_rule_type.UserListRuleTypeEnum.UserListRuleType = (
        proto.Field(
            proto.ENUM,
            number=1,
            enum=user_list_rule_type.UserListRuleTypeEnum.UserListRuleType,
        )
    )
    rule_item_groups: MutableSequence["UserListRuleItemGroupInfo"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="UserListRuleItemGroupInfo",
        )
    )


class UserListRuleItemGroupInfo(proto.Message):
    r"""A group of rule items.

    Attributes:
        rule_items (MutableSequence[google.ads.googleads.v20.common.types.UserListRuleItemInfo]):
            Rule items that will be grouped together based on rule_type.
    """

    rule_items: MutableSequence["UserListRuleItemInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="UserListRuleItemInfo",
    )


class UserListRuleItemInfo(proto.Message):
    r"""An atomic rule item.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Rule variable name. It should match the corresponding key
            name fired by the pixel. A name must begin with US-ascii
            letters or underscore or UTF8 code that is greater than 127
            and consist of US-ascii letters or digits or underscore or
            UTF8 code that is greater than 127. For websites, there are
            two built-in variable URL (name = 'url__') and referrer URL
            (name = 'ref_url__'). This field must be populated when
            creating a new rule item.

            This field is a member of `oneof`_ ``_name``.
        number_rule_item (google.ads.googleads.v20.common.types.UserListNumberRuleItemInfo):
            An atomic rule item composed of a number
            operation.

            This field is a member of `oneof`_ ``rule_item``.
        string_rule_item (google.ads.googleads.v20.common.types.UserListStringRuleItemInfo):
            An atomic rule item composed of a string
            operation.

            This field is a member of `oneof`_ ``rule_item``.
        date_rule_item (google.ads.googleads.v20.common.types.UserListDateRuleItemInfo):
            An atomic rule item composed of a date
            operation.

            This field is a member of `oneof`_ ``rule_item``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    number_rule_item: "UserListNumberRuleItemInfo" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="rule_item",
        message="UserListNumberRuleItemInfo",
    )
    string_rule_item: "UserListStringRuleItemInfo" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="rule_item",
        message="UserListStringRuleItemInfo",
    )
    date_rule_item: "UserListDateRuleItemInfo" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="rule_item",
        message="UserListDateRuleItemInfo",
    )


class UserListDateRuleItemInfo(proto.Message):
    r"""A rule item composed of a date operation.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        operator (google.ads.googleads.v20.enums.types.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator):
            Date comparison operator.
            This field is required and must be populated
            when creating new date rule item.
        value (str):
            String representing date value to be compared
            with the rule variable. Supported date format is
            YYYY-MM-DD. Times are reported in the customer's
            time zone.

            This field is a member of `oneof`_ ``_value``.
        offset_in_days (int):
            The relative date value of the right hand
            side denoted by number of days offset from now.
            The value field will override this field when
            both are present.

            This field is a member of `oneof`_ ``_offset_in_days``.
    """

    operator: (
        user_list_date_rule_item_operator.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_date_rule_item_operator.UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator,
    )
    value: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    offset_in_days: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )


class UserListNumberRuleItemInfo(proto.Message):
    r"""A rule item composed of a number operation.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        operator (google.ads.googleads.v20.enums.types.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator):
            Number comparison operator.
            This field is required and must be populated
            when creating a new number rule item.
        value (float):
            Number value to be compared with the
            variable. This field is required and must be
            populated when creating a new number rule item.

            This field is a member of `oneof`_ ``_value``.
    """

    operator: (
        user_list_number_rule_item_operator.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_number_rule_item_operator.UserListNumberRuleItemOperatorEnum.UserListNumberRuleItemOperator,
    )
    value: float = proto.Field(
        proto.DOUBLE,
        number=3,
        optional=True,
    )


class UserListStringRuleItemInfo(proto.Message):
    r"""A rule item composed of a string operation.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        operator (google.ads.googleads.v20.enums.types.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator):
            String comparison operator.
            This field is required and must be populated
            when creating a new string rule item.
        value (str):
            The right hand side of the string rule item.
            For URLs or referrer URLs, the value can not
            contain illegal URL chars such as newlines,
            quotes, tabs, or parentheses. This field is
            required and must be populated when creating a
            new string rule item.

            This field is a member of `oneof`_ ``_value``.
    """

    operator: (
        user_list_string_rule_item_operator.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_string_rule_item_operator.UserListStringRuleItemOperatorEnum.UserListStringRuleItemOperator,
    )
    value: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )


class FlexibleRuleOperandInfo(proto.Message):
    r"""Flexible rule that wraps the common rule and a lookback
    window.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        rule (google.ads.googleads.v20.common.types.UserListRuleInfo):
            List of rule item groups that defines this
            rule. Rule item groups are grouped together.
        lookback_window_days (int):
            Lookback window for this rule in days. From
            now until X days ago.

            This field is a member of `oneof`_ ``_lookback_window_days``.
    """

    rule: "UserListRuleInfo" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="UserListRuleInfo",
    )
    lookback_window_days: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class FlexibleRuleUserListInfo(proto.Message):
    r"""Flexible rule representation of visitors with one or multiple
    actions. The flexible user list is defined by two lists of operands
    – inclusive_operands and exclusive_operands; each operand represents
    a set of users based on actions they took in a given timeframe.
    These lists of operands are combined with the AND_NOT operator, so
    that users represented by the inclusive operands are included in the
    user list, minus the users represented by the exclusive operands.

    Attributes:
        inclusive_rule_operator (google.ads.googleads.v20.enums.types.UserListFlexibleRuleOperatorEnum.UserListFlexibleRuleOperator):
            Operator that defines how the inclusive
            operands are combined.
        inclusive_operands (MutableSequence[google.ads.googleads.v20.common.types.FlexibleRuleOperandInfo]):
            Rules representing users that should be included in the user
            list. These are located on the left side of the AND_NOT
            operator, and joined together by either AND/OR as specified
            by the inclusive_rule_operator.
        exclusive_operands (MutableSequence[google.ads.googleads.v20.common.types.FlexibleRuleOperandInfo]):
            Rules representing users that should be excluded from the
            user list. These are located on the right side of the
            AND_NOT operator, and joined together by OR.
    """

    inclusive_rule_operator: (
        user_list_flexible_rule_operator.UserListFlexibleRuleOperatorEnum.UserListFlexibleRuleOperator
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_flexible_rule_operator.UserListFlexibleRuleOperatorEnum.UserListFlexibleRuleOperator,
    )
    inclusive_operands: MutableSequence["FlexibleRuleOperandInfo"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="FlexibleRuleOperandInfo",
        )
    )
    exclusive_operands: MutableSequence["FlexibleRuleOperandInfo"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="FlexibleRuleOperandInfo",
        )
    )


class RuleBasedUserListInfo(proto.Message):
    r"""Representation of a userlist that is generated by a rule.

    Attributes:
        prepopulation_status (google.ads.googleads.v20.enums.types.UserListPrepopulationStatusEnum.UserListPrepopulationStatus):
            The status of pre-population. The field is
            default to NONE if not set which means the
            previous users will not be considered. If set to
            REQUESTED, past site visitors or app users who
            match the list definition will be included in
            the list (works on the Display Network only).
            This will only add past users from within the
            last 30 days, depending on the list's membership
            duration and the date when the remarketing tag
            is added. The status will be updated to FINISHED
            once request is processed, or FAILED if the
            request fails.
        flexible_rule_user_list (google.ads.googleads.v20.common.types.FlexibleRuleUserListInfo):
            Flexible rule representation of visitors with one or
            multiple actions. The flexible user list is defined by two
            lists of operands – inclusive_operands and
            exclusive_operands; each operand represents a set of users
            based on actions they took in a given timeframe. These lists
            of operands are combined with the AND_NOT operator, so that
            users represented by the inclusive operands are included in
            the user list, minus the users represented by the exclusive
            operands.
    """

    prepopulation_status: (
        user_list_prepopulation_status.UserListPrepopulationStatusEnum.UserListPrepopulationStatus
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_prepopulation_status.UserListPrepopulationStatusEnum.UserListPrepopulationStatus,
    )
    flexible_rule_user_list: "FlexibleRuleUserListInfo" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="FlexibleRuleUserListInfo",
    )


class LogicalUserListInfo(proto.Message):
    r"""Represents a user list that is a custom combination of user
    lists.

    Attributes:
        rules (MutableSequence[google.ads.googleads.v20.common.types.UserListLogicalRuleInfo]):
            Logical list rules that define this user
            list. The rules are defined as a logical
            operator (ALL/ANY/NONE) and a list of user
            lists. All the rules are ANDed when they are
            evaluated.

            Required for creating a logical user list.
    """

    rules: MutableSequence["UserListLogicalRuleInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="UserListLogicalRuleInfo",
    )


class UserListLogicalRuleInfo(proto.Message):
    r"""A user list logical rule. A rule has a logical operator
    (and/or/not) and a list of user lists as operands.

    Attributes:
        operator (google.ads.googleads.v20.enums.types.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator):
            The logical operator of the rule.
        rule_operands (MutableSequence[google.ads.googleads.v20.common.types.LogicalUserListOperandInfo]):
            The list of operands of the rule.
    """

    operator: (
        user_list_logical_rule_operator.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=user_list_logical_rule_operator.UserListLogicalRuleOperatorEnum.UserListLogicalRuleOperator,
    )
    rule_operands: MutableSequence["LogicalUserListOperandInfo"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="LogicalUserListOperandInfo",
        )
    )


class LogicalUserListOperandInfo(proto.Message):
    r"""Operand of logical user list that consists of a user list.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        user_list (str):
            Resource name of a user list as an operand.

            This field is a member of `oneof`_ ``_user_list``.
    """

    user_list: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class BasicUserListInfo(proto.Message):
    r"""User list targeting as a collection of conversions or
    remarketing actions.

    Attributes:
        actions (MutableSequence[google.ads.googleads.v20.common.types.UserListActionInfo]):
            Actions associated with this user list.
    """

    actions: MutableSequence["UserListActionInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="UserListActionInfo",
    )


class UserListActionInfo(proto.Message):
    r"""Represents an action type used for building remarketing user
    lists.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        conversion_action (str):
            A conversion action that's not generated from
            remarketing.

            This field is a member of `oneof`_ ``user_list_action``.
        remarketing_action (str):
            A remarketing action.

            This field is a member of `oneof`_ ``user_list_action``.
    """

    conversion_action: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="user_list_action",
    )
    remarketing_action: str = proto.Field(
        proto.STRING,
        number=4,
        oneof="user_list_action",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example fetches the set of all ProductCategoryConstants."""


import argparse
import collections
import sys
from typing import DefaultDict, List, Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.product_category_constant import (
    ProductCategoryConstant,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsStreamRequest,
    SearchGoogleAdsStreamResponse,
)


class Category:
    def __init__(
        self,
        localized_name: Optional[str] = None,
        resource_name: Optional[str] = None,
        children: Optional[List["Category"]] = None,
    ) -> None:
        self.localized_name: Optional[str] = localized_name
        self.resource_name: Optional[str] = resource_name
        if children is None:
            self.children: List["Category"] = []
        else:
            self.children: List["Category"] = children


def display_categories(categories: List[Category], prefix: str = "") -> None:
    """Recursively prints out each category and its children.

    Args:
      categories: the map of categories to print
      prefix: the string to print at the beginning of each line of output

    Returns: None
    """
    for category in categories:
        print(f"{prefix} {category.localized_name} [{category.resource_name}]")
        if category.children:
            display_categories(
                category.children, prefix=f"{prefix} {category.localized_name}"
            )


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """Fetches the set of valid ProductBiddingCategories."""

    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    query: str = """
        SELECT
            product_category_constant.localizations,
            product_category_constant.product_category_constant_parent
        FROM product_category_constant"""

    search_request: SearchGoogleAdsStreamRequest = client.get_type(
        "SearchGoogleAdsStreamRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query
    stream: SearchGoogleAdsStreamResponse = ga_service.search_stream(
        search_request
    )

    all_categories: DefaultDict[str, Category] = collections.defaultdict(
        lambda: Category()
    )

    # Creates a map of top level categories.
    root_categories: List[Category] = []

    for batch in stream:
        for row in batch.results:
            # Gets the product category constant from the row.
            product_category: ProductCategoryConstant = (
                row.product_category_constant
            )

            localized_name: str = ""
            for localization in product_category.localizations:
                region: str = localization.region_code
                lang: str = localization.language_code
                if region == "US" and lang == "en":
                    # Gets the name from the product category localization.
                    localized_name = localization.value
                    break

            category: Category = Category(
                localized_name=localized_name,
                resource_name=product_category.resource_name,
            )

            # Ensure resource_name is not None before using as a dict key
            if category.resource_name:
                all_categories[category.resource_name] = category

            parent_resource_name: Optional[str] = None
            if product_category.product_category_constant_parent:
                parent_resource_name = (
                    product_category.product_category_constant_parent
                )

            # Links the category to the parent category if any.
            if parent_resource_name:
                # Adds the category as a child category of the parent
                # category.
                all_categories[parent_resource_name].children.append(category)
            elif (
                category.resource_name
            ):  # Ensure it's not None before adding to root
                # Otherwise adds the category as a root category.
                root_categories.append(category)

    display_categories(root_categories)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Get Product Bidding Category Constant"
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "AdServingOptimizationStatusEnum",
    },
)


class AdServingOptimizationStatusEnum(proto.Message):
    r"""Possible ad serving statuses of a campaign."""

    class AdServingOptimizationStatus(proto.Enum):
        r"""Enum describing possible serving statuses.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            OPTIMIZE (2):
                Ad serving is optimized based on CTR for the
                campaign.
            CONVERSION_OPTIMIZE (3):
                Ad serving is optimized based on CTR \* Conversion for the
                campaign. If the campaign is not in the conversion optimizer
                bidding strategy, it will default to OPTIMIZED.
            ROTATE (4):
                Ads are rotated evenly for 90 days, then
                optimized for clicks.
            ROTATE_INDEFINITELY (5):
                Show lower performing ads more evenly with
                higher performing ads, and do not optimize.
            UNAVAILABLE (6):
                Ad serving optimization status is not
                available.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OPTIMIZE = 2
        CONVERSION_OPTIMIZE = 3
        ROTATE = 4
        ROTATE_INDEFINITELY = 5
        UNAVAILABLE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.resources.types import (
    experiment as gagr_experiment,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateExperimentsRequest",
        "ExperimentOperation",
        "MutateExperimentsResponse",
        "MutateExperimentResult",
        "EndExperimentRequest",
        "ListExperimentAsyncErrorsRequest",
        "ListExperimentAsyncErrorsResponse",
        "GraduateExperimentRequest",
        "CampaignBudgetMapping",
        "ScheduleExperimentRequest",
        "ScheduleExperimentMetadata",
        "PromoteExperimentRequest",
        "PromoteExperimentMetadata",
    },
)


class MutateExperimentsRequest(proto.Message):
    r"""Request message for
    [ExperimentService.MutateExperiments][google.ads.googleads.v19.services.ExperimentService.MutateExperiments].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            experiments are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.ExperimentOperation]):
            Required. The list of operations to perform
            on individual experiments.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["ExperimentOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="ExperimentOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class ExperimentOperation(proto.Message):
    r"""A single operation on an experiment.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v19.resources.types.Experiment):
            Create operation

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.Experiment):
            Update operation: The experiment is expected
            to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: The experiment is expected to have a valid
            resource name, in this format:

            ``customers/{customer_id}/experiments/{campaign_experiment_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: gagr_experiment.Experiment = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_experiment.Experiment,
    )
    update: gagr_experiment.Experiment = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_experiment.Experiment,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateExperimentsResponse(proto.Message):
    r"""Response message for experiment mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateExperimentResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateExperimentResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateExperimentResult",
    )


class MutateExperimentResult(proto.Message):
    r"""The result for the campaign experiment mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class EndExperimentRequest(proto.Message):
    r"""Request message for
    [ExperimentService.EndExperiment][google.ads.googleads.v19.services.ExperimentService.EndExperiment].

    Attributes:
        experiment (str):
            Required. The resource name of the campaign
            experiment to end.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    experiment: str = proto.Field(
        proto.STRING,
        number=1,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class ListExperimentAsyncErrorsRequest(proto.Message):
    r"""Request message for
    [ExperimentService.ListExperimentAsyncErrors][google.ads.googleads.v19.services.ExperimentService.ListExperimentAsyncErrors].

    Attributes:
        resource_name (str):
            Required. The name of the experiment from
            which to retrieve the async errors.
        page_token (str):
            Token of the page to retrieve. If not specified, the first
            page of results will be returned. Use the value obtained
            from ``next_page_token`` in the previous response in order
            to request the next page of results.
        page_size (int):
            Number of elements to retrieve in a single
            page. When a page request is too large, the
            server may decide to further limit the number of
            returned resources. The maximum page size is
            1000.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )


class ListExperimentAsyncErrorsResponse(proto.Message):
    r"""Response message for
    [ExperimentService.ListExperimentAsyncErrors][google.ads.googleads.v19.services.ExperimentService.ListExperimentAsyncErrors].

    Attributes:
        errors (MutableSequence[google.rpc.status_pb2.Status]):
            details of the errors when performing the
            asynchronous operation.
        next_page_token (str):
            Pagination token used to retrieve the next page of results.
            Pass the content of this string as the ``page_token``
            attribute of the next request. ``next_page_token`` is not
            returned for the last page.
    """

    @property
    def raw_page(self):
        return self

    errors: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GraduateExperimentRequest(proto.Message):
    r"""Request message for
    [ExperimentService.GraduateExperiment][google.ads.googleads.v19.services.ExperimentService.GraduateExperiment].

    Attributes:
        experiment (str):
            Required. The experiment to be graduated.
        campaign_budget_mappings (MutableSequence[google.ads.googleads.v19.services.types.CampaignBudgetMapping]):
            Required. List of campaign budget mappings
            for graduation. Each campaign that appears here
            will graduate, and will be assigned a new budget
            that is paired with it in the mapping. The
            maximum size is one.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    experiment: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign_budget_mappings: MutableSequence["CampaignBudgetMapping"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CampaignBudgetMapping",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CampaignBudgetMapping(proto.Message):
    r"""The mapping of experiment campaign and budget to be
    graduated.

    Attributes:
        experiment_campaign (str):
            Required. The experiment campaign to
            graduate.
        campaign_budget (str):
            Required. The budget that should be attached
            to the graduating experiment campaign.
    """

    experiment_campaign: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign_budget: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ScheduleExperimentRequest(proto.Message):
    r"""Request message for
    [ExperimentService.ScheduleExperiment][google.ads.googleads.v19.services.ExperimentService.ScheduleExperiment].

    Attributes:
        resource_name (str):
            Required. The scheduled experiment.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class ScheduleExperimentMetadata(proto.Message):
    r"""The metadata of the scheduled experiment.

    Attributes:
        experiment (str):
            Required. The scheduled experiment.
    """

    experiment: str = proto.Field(
        proto.STRING,
        number=1,
    )


class PromoteExperimentRequest(proto.Message):
    r"""Request message for
    [ExperimentService.PromoteExperiment][google.ads.googleads.v19.services.ExperimentService.PromoteExperiment].

    Attributes:
        resource_name (str):
            Required. The resource name of the experiment
            to promote.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class PromoteExperimentMetadata(proto.Message):
    r"""The metadata of the promoted experiment.

    Attributes:
        experiment (str):
            Required. The promoted experiment.
    """

    experiment: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "SmartCampaignStatusEnum",
    },
)


class SmartCampaignStatusEnum(proto.Message):
    r"""A container for an enum that describes Smart campaign
    statuses.

    """

    class SmartCampaignStatus(proto.Enum):
        r"""Smart campaign statuses.

        Values:
            UNSPECIFIED (0):
                The status has not been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            PAUSED (2):
                The campaign was paused.
            NOT_ELIGIBLE (3):
                The campaign is not eligible to serve and has
                issues that may require intervention.
            PENDING (4):
                The campaign is pending the approval of at
                least one ad.
            ELIGIBLE (5):
                The campaign is eligible to serve.
            REMOVED (6):
                The campaign has been removed.
            ENDED (7):
                The campaign has ended.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PAUSED = 2
        NOT_ELIGIBLE = 3
        PENDING = 4
        ELIGIBLE = 5
        REMOVED = 6
        ENDED = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.resources.types import (
    customer_user_access_invitation,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "MutateCustomerUserAccessInvitationRequest",
        "CustomerUserAccessInvitationOperation",
        "MutateCustomerUserAccessInvitationResponse",
        "MutateCustomerUserAccessInvitationResult",
    },
)


class MutateCustomerUserAccessInvitationRequest(proto.Message):
    r"""Request message for
    [CustomerUserAccessInvitationService.MutateCustomerUserAccessInvitation][google.ads.googleads.v20.services.CustomerUserAccessInvitationService.MutateCustomerUserAccessInvitation]

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose access
            invitation is being modified.
        operation (google.ads.googleads.v20.services.types.CustomerUserAccessInvitationOperation):
            Required. The operation to perform on the
            access invitation
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "CustomerUserAccessInvitationOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="CustomerUserAccessInvitationOperation",
    )


class CustomerUserAccessInvitationOperation(proto.Message):
    r"""A single operation (create or remove) on customer user access
    invitation.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v20.resources.types.CustomerUserAccessInvitation):
            Create operation: No resource name is
            expected for the new access invitation.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the revoke invitation
            is expected, in this format:

            ``customers/{customer_id}/customerUserAccessInvitations/{invitation_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    create: customer_user_access_invitation.CustomerUserAccessInvitation = (
        proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="operation",
            message=customer_user_access_invitation.CustomerUserAccessInvitation,
        )
    )
    remove: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="operation",
    )


class MutateCustomerUserAccessInvitationResponse(proto.Message):
    r"""Response message for access invitation mutate.

    Attributes:
        result (google.ads.googleads.v20.services.types.MutateCustomerUserAccessInvitationResult):
            Result for the mutate.
    """

    result: "MutateCustomerUserAccessInvitationResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="MutateCustomerUserAccessInvitationResult",
    )


class MutateCustomerUserAccessInvitationResult(proto.Message):
    r"""The result for the access invitation mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import combined_audience_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "CombinedAudience",
    },
)


class CombinedAudience(proto.Message):
    r"""Describe a resource for combined audiences which includes
    different audiences.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the combined audience.
            Combined audience names have the form:

            ``customers/{customer_id}/combinedAudience/{combined_audience_id}``
        id (int):
            Output only. ID of the combined audience.
        status (google.ads.googleads.v21.enums.types.CombinedAudienceStatusEnum.CombinedAudienceStatus):
            Output only. Status of this combined
            audience. Indicates whether the combined
            audience is enabled or removed.
        name (str):
            Output only. Name of the combined audience.
            It should be unique across all combined
            audiences.
        description (str):
            Output only. Description of this combined
            audience.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    status: (
        combined_audience_status.CombinedAudienceStatusEnum.CombinedAudienceStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=combined_audience_status.CombinedAudienceStatusEnum.CombinedAudienceStatus,
    )
    name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    description: str = proto.Field(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v22.resources.types import campaign as gagr_campaign
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateCampaignsRequest",
        "CampaignOperation",
        "MutateCampaignsResponse",
        "MutateCampaignResult",
        "EnablePMaxBrandGuidelinesRequest",
        "EnableOperation",
        "BrandCampaignAssets",
        "EnablePMaxBrandGuidelinesResponse",
        "EnablementResult",
    },
)


class MutateCampaignsRequest(proto.Message):
    r"""Request message for
    [CampaignService.MutateCampaigns][google.ads.googleads.v22.services.CampaignService.MutateCampaigns].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            campaigns are being modified.
        operations (MutableSequence[google.ads.googleads.v22.services.types.CampaignOperation]):
            Required. The list of operations to perform
            on individual campaigns.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v22.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CampaignOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CampaignOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class CampaignOperation(proto.Message):
    r"""A single operation (create, update, remove) on a campaign.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v22.resources.types.Campaign):
            Create operation: No resource name is
            expected for the new campaign.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v22.resources.types.Campaign):
            Update operation: The campaign is expected to
            have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed campaign
            is expected, in this format:

            ``customers/{customer_id}/campaigns/{campaign_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: gagr_campaign.Campaign = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_campaign.Campaign,
    )
    update: gagr_campaign.Campaign = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_campaign.Campaign,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateCampaignsResponse(proto.Message):
    r"""Response message for campaign mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v22.services.types.MutateCampaignResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateCampaignResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateCampaignResult",
    )


class MutateCampaignResult(proto.Message):
    r"""The result for the campaign mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        campaign (google.ads.googleads.v22.resources.types.Campaign):
            The mutated campaign with only mutable fields after mutate.
            The field will only be returned when response_content_type
            is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: gagr_campaign.Campaign = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_campaign.Campaign,
    )


class EnablePMaxBrandGuidelinesRequest(proto.Message):
    r"""Request to enable Brand Guidelines for a Performance Max
    campaign.

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            campaigns are being enabled.
        operations (MutableSequence[google.ads.googleads.v22.services.types.EnableOperation]):
            Required. The list of individual campaign
            operations. A maximum of 10 enable operations
            can be executed in a request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["EnableOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="EnableOperation",
    )


class EnableOperation(proto.Message):
    r"""A single enable operation of a campaign.

    Attributes:
        campaign (str):
            Required. The resource name of the campaign
            to enable.
        auto_populate_brand_assets (bool):
            Required. The switch to automatically populate
            top-performing brand assets. This field is required. If
            true, top-performing brand assets will be automatically
            populated. If false, the brand_assets field is required.
        brand_assets (google.ads.googleads.v22.services.types.BrandCampaignAssets):
            Optional. The brand assets linked to the campaign. This
            field is required when the value of
            auto_populate_brand_assets is false.
        final_uri_domain (str):
            Optional. The domain of the final uri.
        main_color (str):
            Optional. Hex code representation of the main brand color,
            for example #00ff00. main_color is required when accent
            color is specified.
        accent_color (str):
            Optional. Hex code representation of the accent brand color,
            for example #00ff00. accent_color is required when
            main_color is specified.
        font_family (str):
            Optional. The font family is specified as a
            string, and must be one of the following: "Open
            Sans", "Roboto", "Roboto Slab", "Montserrat",
            "Poppins", "Lato", "Oswald", or "Playfair
            Display".
    """

    campaign: str = proto.Field(
        proto.STRING,
        number=1,
    )
    auto_populate_brand_assets: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    brand_assets: "BrandCampaignAssets" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="BrandCampaignAssets",
    )
    final_uri_domain: str = proto.Field(
        proto.STRING,
        number=4,
    )
    main_color: str = proto.Field(
        proto.STRING,
        number=5,
    )
    accent_color: str = proto.Field(
        proto.STRING,
        number=6,
    )
    font_family: str = proto.Field(
        proto.STRING,
        number=7,
    )


class BrandCampaignAssets(proto.Message):
    r"""Assets linked at the campaign level. A business_name and at least
    one logo_asset are required.

    Attributes:
        business_name_asset (str):
            Required. The resource name of the business
            name text asset.
        logo_asset (MutableSequence[str]):
            Required. The resource name of square logo
            assets.
        landscape_logo_asset (MutableSequence[str]):
            Optional. The resource name of landscape logo
            assets.
    """

    business_name_asset: str = proto.Field(
        proto.STRING,
        number=1,
    )
    logo_asset: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    landscape_logo_asset: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class EnablePMaxBrandGuidelinesResponse(proto.Message):
    r"""Brand Guidelines campaign enablement response.

    Attributes:
        results (MutableSequence[google.ads.googleads.v22.services.types.EnablementResult]):
            Campaign enablement results per campaign.
    """

    results: MutableSequence["EnablementResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="EnablementResult",
    )


class EnablementResult(proto.Message):
    r"""A single enablement result of a campaign.

    Attributes:
        campaign (str):
            This indicates the campaign for which
            enablement was tried, regardless of the outcome.
        enablement_error (google.rpc.status_pb2.Status):
            Details of the error when enablement fails.
    """

    campaign: str = proto.Field(
        proto.STRING,
        number=1,
    )
    enablement_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "UserListCustomerTypeErrorEnum",
    },
)


class UserListCustomerTypeErrorEnum(proto.Message):
    r"""Container for enum describing possible user list customer
    type errors.

    """

    class UserListCustomerTypeError(proto.Enum):
        r"""Enum describing possible user list customer type errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CONFLICTING_CUSTOMER_TYPES (2):
                Cannot add the conflicting customer types to
                the same user list. Conflicting labels:

                1. Purchasers - Converted Leads
                2. Purchasers - Qualified Leads
                3. Purchasers - Cart Abandoners
                4. Qualified Leads - Converted Leads
                5. Disengaged customers - Converted Leads
                6. Disengaged customers - Qualified Leads
                7. Disengaged customers- Cart Abandoners
            NO_ACCESS_TO_USER_LIST (3):
                The account does not have access to the user
                list.
            USERLIST_NOT_ELIGIBLE (4):
                The given user list is not eligible for applying customer
                types. The user list must belong to one of the following
                types: CRM_BASED, RULE_BASED, ADVERTISER_DATA_MODEL_BASED,
                GCN.
            CONVERSION_TRACKING_NOT_ENABLED_OR_NOT_MCC_MANAGER_ACCOUNT (5):
                To edit the user list customer type,
                conversion tracking must be enabled in your
                account. If cross-tracking is enabled, your
                account must be a MCC manager account to modify
                user list customer types. More info at
                https://support.google.com/google-ads/answer/3030657
            TOO_MANY_USER_LISTS_FOR_THE_CUSTOMER_TYPE (6):
                Too many user lists for the customer type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONFLICTING_CUSTOMER_TYPES = 2
        NO_ACCESS_TO_USER_LIST = 3
        USERLIST_NOT_ELIGIBLE = 4
        CONVERSION_TRACKING_NOT_ENABLED_OR_NOT_MCC_MANAGER_ACCOUNT = 5
        TOO_MANY_USER_LISTS_FOR_THE_CUSTOMER_TYPE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.common.types import asset_set_types
from google.ads.googleads.v20.enums.types import asset_set_status
from google.ads.googleads.v20.enums.types import asset_set_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "AssetSet",
    },
)


class AssetSet(proto.Message):
    r"""An asset set representing a collection of assets.
    Use AssetSetAsset to link an asset to the asset set.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        id (int):
            Output only. The ID of the asset set.
        resource_name (str):
            Immutable. The resource name of the asset set. Asset set
            resource names have the form:

            ``customers/{customer_id}/assetSets/{asset_set_id}``
        name (str):
            Required. Name of the asset set. Required. It
            must have a minimum length of 1 and maximum
            length of 128.
        type_ (google.ads.googleads.v20.enums.types.AssetSetTypeEnum.AssetSetType):
            Required. Immutable. The type of the asset
            set. Required.
        status (google.ads.googleads.v20.enums.types.AssetSetStatusEnum.AssetSetStatus):
            Output only. The status of the asset set.
            Read-only.
        merchant_center_feed (google.ads.googleads.v20.resources.types.AssetSet.MerchantCenterFeed):
            Merchant ID and Feed Label from Google
            Merchant Center.
        location_group_parent_asset_set_id (int):
            Immutable. Parent asset set id for the asset
            set where the elements of this asset set come
            from. For example: the sync level location
            AssetSet id where the the elements in
            LocationGroup AssetSet come from. This field is
            required and only applicable for Location Group
            typed AssetSet.
        hotel_property_data (google.ads.googleads.v20.resources.types.AssetSet.HotelPropertyData):
            Output only. For Performance Max for travel
            goals campaigns with a Hotel Center account
            link. Read-only.
        location_set (google.ads.googleads.v20.common.types.LocationSet):
            Location asset set data. This will be used for sync level
            location set. This can only be set if AssetSet's type is
            LOCATION_SYNC.

            This field is a member of `oneof`_ ``asset_set_source``.
        business_profile_location_group (google.ads.googleads.v20.common.types.BusinessProfileLocationGroup):
            Business Profile location group asset set
            data.

            This field is a member of `oneof`_ ``asset_set_source``.
        chain_location_group (google.ads.googleads.v20.common.types.ChainLocationGroup):
            Represents information about a Chain dynamic location group.
            Only applicable if the sync level AssetSet's type is
            LOCATION_SYNC and sync source is chain.

            This field is a member of `oneof`_ ``asset_set_source``.
    """

    class MerchantCenterFeed(proto.Message):
        r"""Merchant ID and Feed Label from Google Merchant Center.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            merchant_id (int):
                Required. Merchant ID from Google Merchant
                Center
            feed_label (str):
                Optional. Feed Label from Google Merchant
                Center.

                This field is a member of `oneof`_ ``_feed_label``.
        """

        merchant_id: int = proto.Field(
            proto.INT64,
            number=1,
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )

    class HotelPropertyData(proto.Message):
        r"""For Performance Max for travel goals campaigns with a Hotel
        Center account link. Read-only.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            hotel_center_id (int):
                Output only. The hotel center ID of the
                partner.

                This field is a member of `oneof`_ ``_hotel_center_id``.
            partner_name (str):
                Output only. Name of the hotel partner.

                This field is a member of `oneof`_ ``_partner_name``.
        """

        hotel_center_id: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )
        partner_name: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )

    id: int = proto.Field(
        proto.INT64,
        number=6,
    )
    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    type_: asset_set_type.AssetSetTypeEnum.AssetSetType = proto.Field(
        proto.ENUM,
        number=3,
        enum=asset_set_type.AssetSetTypeEnum.AssetSetType,
    )
    status: asset_set_status.AssetSetStatusEnum.AssetSetStatus = proto.Field(
        proto.ENUM,
        number=4,
        enum=asset_set_status.AssetSetStatusEnum.AssetSetStatus,
    )
    merchant_center_feed: MerchantCenterFeed = proto.Field(
        proto.MESSAGE,
        number=5,
        message=MerchantCenterFeed,
    )
    location_group_parent_asset_set_id: int = proto.Field(
        proto.INT64,
        number=10,
    )
    hotel_property_data: HotelPropertyData = proto.Field(
        proto.MESSAGE,
        number=11,
        message=HotelPropertyData,
    )
    location_set: asset_set_types.LocationSet = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="asset_set_source",
        message=asset_set_types.LocationSet,
    )
    business_profile_location_group: (
        asset_set_types.BusinessProfileLocationGroup
    ) = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="asset_set_source",
        message=asset_set_types.BusinessProfileLocationGroup,
    )
    chain_location_group: asset_set_types.ChainLocationGroup = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="asset_set_source",
        message=asset_set_types.ChainLocationGroup,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
